"""filter a HTML stream."""
# -*- coding: iso-8859-1 -*-
# Copyright (C) 2000-2003  Bastian Kleineidam
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

__version__ = "$Revision: 1.95 $"[11:-2]
__date__    = "$Date: 2003/07/05 09:45:31 $"[7:-2]

import re
from wc.filter import FILTER_RESPONSE_MODIFY
from wc.filter import compileMime, compileRegex
from wc.filter.Filter import Filter
from wc.filter.HtmlParser import FilterHtmlParser

# which filter stages this filter applies to (see filter/__init__.py)
orders = [FILTER_RESPONSE_MODIFY]
# which rule types this filter applies to (see Rules.py)
# all rules of these types get added with Filter.addrule()
rulenames = ['rewrite', 'nocomments', 'javascript', 'pics']
mimelist = map(compileMime, ['text/html'])

class Rewriter (Filter):
    """This filter can rewrite HTML tags. It uses a parser class."""

    def __init__ (self, mimelist):
        super(Rewriter, self).__init__(mimelist)
        self.comments = 1


    def addrule (self, rule):
        super(Rewriter, self).addrule(rule)
        compileRegex(rule, "matchurl")
        compileRegex(rule, "dontmatchurl")
        if rule.get_name()=='rewrite':
            compileRegex(rule, "enclosed")
            for key,val in rule.attrs.items():
                rule.attrs[key] = re.compile(rule.attrs[key])
        elif rule.get_name()=='nocomments':
            self.comments = 0


    def filter (self, data, **attrs):
        if not attrs.has_key('filter'): return data
        p = attrs['filter']
        p.feed(data)
        return p.flushbuf()


    def finish (self, data, **attrs):
        if not attrs.has_key('filter'): return data
        p = attrs['filter']
        # feed even if data is empty
        p.feed(data)
        p.flush()
        p.buf2data()
        return p.flushbuf()


    def getAttrs (self, headers, url):
        """We need a separate filter instance for stateful filtering"""
        rewrites = []
        pics = []
        # look if headers already have PICS label info
        picsheader = headers.has_key('PICS-Label')
        opts = {'comments': 1, 'javascript': 0}
        for rule in self.rules:
            if not rule.appliesTo(url): continue
            if rule.get_name()=='rewrite':
                rewrites.append(rule)
            elif rule.get_name()=='nocomments':
                opts['comments'] = 0
            elif rule.get_name()=='javascript':
                opts['javascript'] = 1
            elif rule.get_name()=='pics' and not picsheader:
                pics.append(rule)
        # generate the HTML filter
        return {'filter': FilterHtmlParser(rewrites, pics, url, **opts)}
